include "Vram.inc"
include "VVram.inc"
include "Chars.inc"

ext MonoPattern, ColorPattern, SpritePattern
ext VVram_

SequencerPort equ 3c4h
MapMaskRegister equ 02h

GraphicsPort equ 3ceh
EnableSetResetRegister equ 01h
ColorCompareRegister equ 02h
DataRotateRegister equ 03h
ModeRegister equ 05h
BitMaskRegister equ 08h

CharPatternSize equ PlaneCount*CharHeight
SpritePatternSize equ CharPatternSize*4
Top equ VramRowSize*0


dseg
Backup:
	defs VVramWidth*(VVramHeight)
CharRam: public CharRam
    defs CharPatternSize*Char_Wall

cseg
ColorSource:
defb 64,0fh ;Ascii
defb 4,0fh ;M
defb 4,0dh ;A
defb 4,0eh ;Z
defb 4,0ch ;Y
defb 0
InitVram: public InitVram
    mov ax,000dh | int 10h ; Set video mode
    mov ax,0500h | int 10h ; Select active display page

    mov si,MonoPattern
    mov di,CharRam
    mov bx,ColorSource
    do
        mov ch,[bx]
        or ch,ch
    while nz
        inc bx
        mov dl,[bx] | inc bx
        call MakePatternMono
    wend

    mov si,ColorPattern
    mov ch,Char_Wall-Char_Knife
    call MakePatternColor
ret


MakePatternMono: public MakePatternMono
;   di: destination
;   si: source
;   dl: color
;   ch: count
    do
        mov cl,CharHeight
        do
            push cx
                mov al,[si] | inc si
                mov dh,dl
                mov cl,PlaneCount
                do
                    xor ah,ah
                    test dh,01h
                    if nz
                        or ah,al
                    endif
                    test dh,10h
                    if nz
                        not al
                        or ah,al
                        not al
                    endif
                    mov [di],ah | inc di
                    shr dh,1
                    dec cl
                while nz | wend
            pop cx
            dec cl
        while nz | wend
        dec ch
    while nz | wend
ret
MakePatternColor:
;   di: destination
;   si: source
;   ch: count
    do
        mov cl,CharPatternSize
        do
            mov al,[si] | inc si
            mov [di],al | inc di
            dec cl
        while nz | wend
        dec ch
    while nz | wend
ret


; void ClearScreen();
cseg
ClearScreen_: public ClearScreen_
    push ax | push bx | push cx | push dx
        mov dx,SequencerPort
        mov ax,(0fh shl 8) or MapMaskRegister
        out dx,ax

        mov ax,VramSegment | mov es,ax
        xor ax,ax
        xor bx,bx
        mov cx,2000h
        do
            mov [es:bx],ax
            inc bx | inc bx
        wloop

		mov bx,Backup
		mov cx,VVramWidth*VVramHeight/2
        xor ax,ax 
        do
            mov [bx],ax
            inc bx | inc bx
        wloop
    pop dx | pop cx | pop bx | pop ax
ret


cseg
Put1:
;   al: char
;   es:di: vram address
    push cx | push dx | push di
        or al,al
        if z
            mov dx,SequencerPort
            mov ax,MapMaskRegister or (0fh shl 8)
            out dx,ax

            mov cx,CharHeight shl 8
            do
                mov [es:di],cl
                add di,VramWidth
                dec ch
            while nz | wend
            jmp Put1_exit
        endif
        cmp al,Char_Wall
        if z
            mov dx,SequencerPort
            mov ax,MapMaskRegister or (0fh shl 8)
            out dx,ax

            mov dx,GraphicsPort
            mov ax,EnableSetResetRegister or 400h
            out dx,ax

            mov cx,0ffh or (CharHeight shl 8)
            do
                mov [es:di],cl
                add di,VramWidth
                dec ch
            while nz | wend

            mov dx,GraphicsPort
            mov ax,EnableSetResetRegister
            out dx,ax
        else
            mov ah,CharPatternSize | mul ah
            add ax,CharRam
            mov si,ax

            mov dx,SequencerPort
            mov al,MapMaskRegister

            mov ch,CharHeight
            do
                mov ah,01h
                do
                    out dx,ax
                    mov cl,[si] | inc si
                    mov [es:di],cl
                    shl ah,1
                    and ah,0fh
                while nz | wend        
                add di,VramWidth
                dec ch
            while nz | wend
            jmp Put1_exit
        endif
        Put1_exit:
    pop di | pop dx | pop cx
ret


; word Put(word vram, byte c);
cseg
Put_: public Put_
	push ax | push si | push di
	    mov di,ax
        mov ax,VramSegment | mov es,ax
        mov al,dl
        call Put1
	pop di | pop si | pop ax
	add ax,VramStep
ret


; void VVramToVram();
cseg
VVramToVram_: public VVramToVram_
	push ax | push cx | push dx | push bx | push si | push di
		mov bx,VVram_
		mov si,Backup
		mov ax,VramSegment | mov es,ax
        mov di,Top
		mov ch, VVramHeight
		do
			mov cl,VVramWidth
			do 
				mov al,[bx]
				inc bx
				cmp al,[si]
				if ne
					mov [si],al
					push si
						call Put1
					pop si
				endif
				add di,VramStep
				inc si
				dec cl
			while nz | wend
			add di,VramRowSize-VVramWidth*VramStep
			dec ch
		while nz | wend
	pop di | pop si | pop bx | pop dx | pop cx | pop ax
ret
